'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = visitor;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var EXTENSION = '.js';
var SPLIT_CHAR = '/';

var exists = function exists(path) {
  if (_fs2.default.existsSync(path) && _fs2.default.statSync(path).isFile()) return true;else return false;
};

var hasSource = function hasSource(path) {
  if (!path.node.source) return false;
  return true;
};

var getNewPath = function getNewPath(value, filename) {
  var root = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '.';

  if (_path2.default.extname(value) !== '') return null;
  var split = value.split(SPLIT_CHAR);
  var dir = split.slice(-1)[0];
  var newPath = [].concat(_toConsumableArray(split), [dir]).join(SPLIT_CHAR) + EXTENSION;
  if (_path2.default.isAbsolute(newPath) || !newPath.startsWith('.')) {
    var fullAbsolutePath = _path2.default.resolve(root, newPath);
    if (!exists(fullAbsolutePath)) return null;
    return newPath;
  } else {
    var fullPath = _path2.default.resolve(_path2.default.resolve(filename, '..'), newPath);
    if (!exists(fullPath)) return null;
    return newPath;
  }
};

var isRequire = function isRequire(node, t) {
  var args = node.arguments || [];
  if (node.callee.name === 'require' && args.length === 1 && t.isStringLiteral(args[0])) return true;
  return false;
};

var getRootDir = function getRootDir(state) {
  return state.opts.rootDir || 'src';
};

function visitor(_ref) {
  var t = _ref.types;

  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(path, state) {
        var value = path.node.source.value;
        var filename = state.file.opts.filename;

        var rootDir = getRootDir(state);
        var newPath = getNewPath(value, filename, rootDir);
        if (!newPath) return;
        var newSource = t.stringLiteral(value.replace(value, newPath));
        path.node.source = newSource;
      },
      ExportDeclaration: function ExportDeclaration(path, state) {
        if (!hasSource(path)) return;
        var value = path.node.source.value;
        var filename = state.file.opts.filename;

        var rootDir = getRootDir(state);
        var newPath = getNewPath(value, filename, rootDir);
        if (!newPath) return;
        var newSource = t.stringLiteral(value.replace(value, newPath));
        path.node.source = newSource;
      },
      CallExpression: function CallExpression(path, state) {
        var node = path.node;

        if (!isRequire(node, t)) return;
        var value = node.arguments[0].value;
        var filename = state.file.opts.filename;

        var rootDir = getRootDir(state);
        var newPath = getNewPath(value, filename, rootDir);
        if (!newPath) return;
        path.node.arguments[0] = t.stringLiteral(value.replace(value, newPath));
      }
    }
  };
}